# EventdElement
#
# Object representation of an element on the client's page. Mimics a jQuery style API.
#
# Shot Framework - Copyright (c) Jesse Aaron Dunlap <me@jessedunlap.me>
# Licensed under the MIT License. For full licensing information, please
# see LICENSE.md. http://github.com/JesseDunlap/shot/

class EventdElement < EventdObject
	# Instance of EventdClient, which is used to trigger interface emissions
	attr_accessor :client

	# jQuery Selector associated with the element
	attr_accessor :selector

	# Initializer
	#
	# == Attributes
	# * +selector+
	# * +client+

	def initialize(selector, client)
		super()
		@selector = selector
		@client = client
	end

	# Handles any method called, and attempts to turn it into a valid jQuery call

	def method_missing(method, *attributes, &block)
		self.jq method, attributes
		return self
	end

	# Call a jQuery method
	#
	# == Attribtues
	# * +method+
	# * +*attributes+

	def jq(method, *attributes)
		@client.emit "eapi_jq", {
			:selector => @selector,
			:method => method,
			:attributes => attributes
		}
	end

	# Add an event listener
	# 
	# == Attributes
	# * +event+
	# * +&callback+

	def on(event, &callback)
		random_callback_id = SecureRandom.uuid
		
		@client.emit "eapi_event", {
			:selector => @selector,
			:callback => random_callback_id,
			:event => event
		}

		@client.on random_callback_id do
			callback.call
		end

		super(event, callback)
	end
end